/* 
   hardwire.cc
   $Id: hardwire.cc,v 1.1 1999/07/20 15:00:56 vaidya Exp vaidya $
   */

#include <hardwire/hardwire.h>
#include <ip.h>
#include <cmu-trace.h>
#include <stdio.h>

#define CURRENT_TIME    Scheduler::instance().clock()
#define PT_HARDWIRE PT_AODV

// #define DEBUG


/* ======================================================================
   TCL Hooks
   ====================================================================== */
static class HARDWIREHeaderClass : public PacketHeaderClass {
public:
        HARDWIREHeaderClass() : PacketHeaderClass("PacketHeader/HARDWIRE",
                                              HARDWIRE_HDR_LEN) { } 
} class_rtProtoHARDWIRE_hdr;

static class HARDWIREclass : public TclClass {
public:
        HARDWIREclass() : TclClass("Agent/HARDWIRE") {}
        TclObject* create(int argc, const char*const* argv) {
                assert(argc == 5);
                return (new HARDWIRE((nsaddr_t) atoi(argv[4])));
        }
} class_rtProtoHARDWIRE;



/* ===================================================================== */

HARDWIRE::HARDWIRE(nsaddr_t id) : Agent(PT_HARDWIRE)
{
int i;

        index = id;
//        seqno = 1;
//        bid = 1;

        logtarget = 0;
        ifqueue = 0;

//	initialize route table 
//	for (i=0; i<MAXNUM; i++)
//		routetable[i] = (-1);


	/* default works for LAN only */
	for (i=0; i<MAXNUM; i++)
		routetable[i] = i;

	//printf("Hardwire Routing agent instantiated\n");
}

int
HARDWIRE::command(int argc, const char*const* argv)
{
	int dest,nexthop;

        if(argc == 2) {
                Tcl& tcl = Tcl::instance();

                if(strncasecmp(argv[1], "id", 2) == 0) {
                        tcl.resultf("%d", index);
                        return TCL_OK;
                }
                if(strncasecmp(argv[1], "start", 2) == 0) {
                        return TCL_OK;
                }               
        }
        else if(argc == 3) {

                if(strcmp(argv[1], "index") == 0) {
                        index = atoi(argv[2]);
                        return TCL_OK;
                }
                else if(strcmp(argv[1], "log-target") == 0 || strcmp(argv[1], "tracetarget") == 0) {
                        logtarget = (Trace*) TclObject::lookup(argv[2]);
                        if(logtarget == 0)
                                return TCL_ERROR;
                        return TCL_OK;
                }
                else if(strcmp(argv[1], "drop-target") == 0) {
		        int stat = rqueue.command(argc,argv);
			if (stat != TCL_OK) return stat;
			return Agent::command(argc, argv);
                }
                else if(strcmp(argv[1], "if-queue") == 0) {
                        ifqueue = (PriQueue*) TclObject::lookup(argv[2]);
                        if(ifqueue == 0)
                                return TCL_ERROR;
                        return TCL_OK;
                }
	}
	else if (argc == 4) {
                if(strcmp(argv[1], "next-hop") == 0) {
                        dest = atoi(argv[2]);
			nexthop = atoi(argv[3]);
			if (dest>=MAXNUM) {
				printf("HARDWIRE command dest %d larger than MAXNUM %d\n",dest,MAXNUM);
				exit(1);
			}
			routetable[dest] = nexthop;
                        return TCL_OK;
                }

        }
        return Agent::command(argc, argv);
}


void
HARDWIRE::rt_resolve(Packet *p)
{
        struct hdr_cmn *ch = HDR_CMN(p);
        struct hdr_ip *ih = HDR_IP(p);

        /*
         *  Set the transmit failure callback.  That
         *  won't change.
         */
        ch->xmit_failure_ = 0;
        ch->xmit_failure_data_ = 0;
        ch->prev_hop_ = index;

	if (ih->daddr() >=MAXNUM) {
		printf("HARDWIRE command dest %d larger than MAXNUM %d\n",ih->daddr(),MAXNUM);
		exit(1);
	}

	ch->next_hop_ = routetable[ih->daddr()];
	if (ch->next_hop_ < 0) {
		printf("HARDWIRE at %d routetable[%d] is <0\n",
			index,ih->daddr());
		exit(1);
	}
	ch->addr_type() = NS_AF_INET;

	
	//printf("++++++++++ at %d dest = %d  nexthop = %d\n",
	//     index, ih->daddr(), ch->next_hop_);
	
	//target_->recv(p, (Handler*) 0);
	Scheduler::instance().schedule(target_, p, 0.);
}




/* =====================================================================
   Packet Reception Routines
   ===================================================================== */
void
HARDWIRE::recv(Packet *p, Handler*)
{
        struct hdr_cmn *ch = HDR_CMN(p);
        struct hdr_ip *ih = HDR_IP(p);

        assert(initialized());
        assert(p->incoming == 0);


        /*
         *  Must be a packet I'm originating...
         */

	if(ih->saddr() == index && ch->num_forwards() == 0) {
                /*
                 * Add the IP Header
                 */
                ch->size() += IP_HDR_LEN;

                ih->ttl_ = IP_DEF_TTL;
        }
        /*
         *  I received a packet that I sent.  Probably
         *  a routing loop.
         */
        else if(ih->saddr() == index) {
                drop(p, DROP_RTR_ROUTE_LOOP);
                return;
        }
        /*
         *  Packet I'm forwarding...
         */

        rt_resolve(p);
}




