
#default_options

global opt
set opt(chan)	Channel/WirelessChannel
set opt(prop)	Propagation/TwoRayGround
set opt(netif)	Phy/WirelessPhy
set opt(mac)	Mac/802_11
set opt(ifq)	Queue/DropTail/PriQueue
set opt(ll)	LL
set opt(ant)    Antenna/OmniAntenna
set opt(x)	40
set opt(y)	40
set opt(ifqlen)	2	      ;# max packet in ifq
set opt(seed)	0.0
set opt(tr)	out.tr
set opt(lm)     "off"          ;# log movement
set opt(nn)     8
set opt(rp)     "hardwire"
set opt(stop)   5
set opt(dfstype) "linear"
# =====================================================================
# Other default settings

LL set mindelay_		50us
LL set delay_			25us
LL set bandwidth_		0	;# not used
LL set off_prune_		0	;# not used
LL set off_CtrMcast_		0	;# not used

Agent/Null set sport_		0
Agent/Null set dport_		0

Agent/CBR set sport_		0
Agent/CBR set dport_		0

Agent/TCPSink set sport_	0
Agent/TCPSink set dport_	0

Agent/TCP set sport_		0
Agent/TCP set dport_		0
Agent/TCP set packetSize_	1460

Queue/DropTail/PriQueue set Prefer_Routing_Protocols    1

# unity gain, omni-directional antennas
# set up the antennas to be centered in the node and 1.5 meters above it
Antenna/OmniAntenna set X_ 0
Antenna/OmniAntenna set Y_ 0
Antenna/OmniAntenna set Z_ 1.5
Antenna/OmniAntenna set Gt_ 1.0
Antenna/OmniAntenna set Gr_ 1.0

# Initialize the SharedMedia interface with parameters to make
# it work like the 914MHz Lucent WaveLAN DSSS radio interface
Phy/WirelessPhy set CPThresh_ 10.0
Phy/WirelessPhy set CSThresh_ 1.559e-11
Phy/WirelessPhy set RXThresh_ 3.652e-10
Phy/WirelessPhy set Rb_ 2*1e6
Phy/WirelessPhy set Pt_ 0.2818
Phy/WirelessPhy set freq_ 914e+6 
Phy/WirelessPhy set L_ 1.0

# =====================================================================


Class Myclass1

Class Myclass -superclass Myclass1

proc usage { argv0 }  {
	puts "Usage: $argv0"
	puts "\tmandatory arguments:"
	puts "\t\t\[-x MAXX\] \[-y MAXY\]"
	puts "\toptional arguments:"
	puts "\t\t \[-nn nodes\]"
    puts "\t\t\[-seed seed\] \[-stop sec\] \[-tr tracefile\] \[-dfstype dfstype\]\n"
}

proc getopt {argc argv} {
	global opt
	lappend optlist cp nn seed sc stop tr x y

	for {set i 0} {$i < $argc} {incr i} {
		set arg [lindex $argv $i]
		if {[string range $arg 0 0] != "-"} continue

		set name [string range $arg 1 end]
		set opt($name) [lindex $argv [expr $i+1]]
	        puts "opt($name) is $opt($name)"
	}
}

Myclass1 instproc init {} {
    global opt tracefd topo chan prop 
    global node_ god_ 
    $self instvar ns_ testName_
    
    set ns_         [new Simulator]
    set topo	[new Topography]
    set tracefd	[open $opt(tr) w]
    
    $ns_ trace-all $tracefd
    
    #set opt(rp) $testName_
    $topo load_flatgrid $opt(x) $opt(y)
    
    
    puts $tracefd "M 0.0 nn:$opt(nn) x:$opt(x) y:$opt(y) rp:$opt(rp)"
    puts $tracefd "M 0.0  seed:$opt(seed) stop:$opt(stop)"
    puts $tracefd "M 0.0 prop:$opt(prop) ant:$opt(ant)"
}

Myclass instproc init {} {
    global opt node_ god_ chan topo
    $self instvar ns_ testName_

    set opt(rp)         hardwire

    puts "x is $opt(x)"
    puts "y is $opt(y)"
    
    $self next

    #
    # Create God
    #
    
    set god_ [create-god $opt(nn)]
    
    $ns_ node-config -adhocRouting HARDWIRE \
	-llType $opt(ll) \
	-macType $opt(mac) \
	-ifqType $opt(ifq) \
	-ifqLen $opt(ifqlen) \
	-antType $opt(ant) \
	-propType $opt(prop) \
	-phyType $opt(netif) \
	-channelType $opt(chan) \
	-topoInstance $topo \
	-agentTrace ON \
	-routerTrace ON \
	-macTrace OFF \
	-toraDebug OFF \
	-movementTrace OFF
    
    for {set i 0} {$i < $opt(nn) } {incr i} {
	set node_($i) [$ns_ node]
	$node_($i) random-motion 0              ;# disable random motion
    }
    puts "Loading connection pattern..."

    puts "Num of nodes is $opt(nn)"
    
    set j [expr $opt(nn)/2]
    
    for {set i 0} {$i < $j } {incr i} {
	set udp_($i) [new Agent/UDP]
	set n1 [expr $i*2]
	set n2 [expr $n1+1]
	$ns_ attach-agent $node_($n1) $udp_($i)
	set null_($i) [new Agent/Null]
	$ns_ attach-agent $node_($n2) $null_($i)
	set cbr_($i) [new Application/Traffic/CBR]
	$cbr_($i) set random_ 1
	
	set size 512
	$cbr_($i) set interval_ 0.2
	set starttime 0.0
	$cbr_($i) set maxpkts_ 10
	
	#set endtime 5.0
	
	#puts "temporary connection \#$i from $n1 to $n2"
	    
	$cbr_($i) set packetSize_ $size
	$cbr_($i) attach-agent $udp_($i)
	$ns_ connect $udp_($i) $null_($i)
	$ns_ at $starttime "$cbr_($i) start"
	#$ns_ at $endtime "$cbr_($i) stop"
	
    }


    
    for {set i 0} {$i < $j } {incr i} {
	set i1 [expr $i + $j]
	set udp_($i1) [new Agent/UDP]
	set n1 [expr $i*2]
	set n2 [expr $n1+1]
	$ns_ attach-agent $node_($n1) $udp_($i1)
	set null_($i1) [new Agent/Null]
	$ns_ attach-agent $node_($n2) $null_($i1)
	set cbr_($i1) [new Application/Traffic/CBR]
	$cbr_($i1) set random_ 1
	
	set size 512
	
	if {$i == 0} {
	    #set size 512
	    $cbr_($i1) set interval_ 0.0009
	} else {
	    #set size 128
	    $cbr_($i1) set interval_ 0.002
	}
	
	#$cbr_($i1) set interval_ 0.0009
	set starttime 10.0
	#[expr 10.0 + [expr $i/1000.0]]
	$cbr_($i1) set maxpkts_ 5000000
	
	#set endtime 2.0
	    
#	puts "real connection \#$i1 from $n1 to $n2 $size"
	
	$cbr_($i1) set packetSize_ $size
	$cbr_($i1) attach-agent $udp_($i1)
	$ns_ connect $udp_($i1) $null_($i1)
	$ns_ at $starttime "$cbr_($i1) start"
	#$ns_ at $endtime "$cbr_($i1) stop"
    }
    
    
#    source $opt(cp)
    
    puts "Loading scenario file..."

    set org_x 10
    set x 10
    set y 10
    set inc 1
    
    set arr [expr round(sqrt($opt(nn))) ]
    puts "arr is $arr"
    for {set i 0} {$i < $arr } {incr i} {
	for {set j 0} {$j < $arr } {incr j} {
	    set temp1 [ expr [expr $i*$arr] + $j]
	    if {$temp1 < $opt(nn)} {
		$node_($temp1) set X_ $x
		$node_($temp1) set Y_ $y
		$node_($temp1) set Z_ 0
		
		#puts "node $temp1 $x $y"
		set x [expr $x + $inc]
	    }
	}
	set y [expr $y + $inc]
	set x $org_x
    }
    
    
    while {$temp1 < [expr $opt(nn) -1] } {
	set temp1 [expr $temp1 +1]
	$node_($temp1) set X_ $x
	$node_($temp1) set Y_ $y
	$node_($temp1) set Z_ 0
	#puts "node $temp1 $x $y"
	set x [expr $x + $inc]
    }
    
    for {set i 0} {$i < $opt(nn) } {incr i} {
	for {set j [expr $i+1]} {$j < $opt(nn) } {incr j} {
	    $god_ set-dist $i $j 1
	}
    }
    

#    source $opt(sc)
    puts "Load complete..."
    
    $ns_ at $opt(stop) "puts \"NS EXITING...\";"
    $ns_ at $opt(stop).001 "$self finish"
    
}

Myclass instproc run {} {
    $self instvar ns_
    puts "Starting Simulation..."
    $ns_ run
}

Myclass1 instproc finish {} {
	$self instvar ns_
	

    $ns_ flush-trace
    puts "finishing.."
    exit 0
}


proc runtest {} {

    set t [new Myclass]
    $t run
}

getopt $argc $argv

if {$opt(seed) >= 0} { ns-random $opt(seed) }

set xx $opt(nn)
set weight [expr 2.0/[expr $xx+0.0]]
Mac/802_11 set weight_ $weight
puts "nn is $opt(nn) x is $xx weight is $weight"
 
Mac/802_11 set max_count_inc 10000
Mac/802_11 set scaling_factor 0.02

set xx $opt(nn)
set weight [expr 2.0/[expr $xx+0.0]]
set cw [expr 12.85 / $weight]



if {$opt(dfstype) == "linear"} {
#Mac/802_11 set vtvar 0
puts "DFStype is LINEAR"

} elseif {$opt(dfstype) == "exp"} {

Mac/802_11 set k2_multiplier 0.002
Mac/802_11 set exp_threshold 80.0
puts "DFStype is EXP"

} elseif {$opt(dfstype) == "sqrt"} {

Mac/802_11 set exp_threshold 80.0
Mac/802_11 set k1_multiplier 8.9
puts "DFStype is SQRT"

} elseif {$opt(dfstype) == "aggcoldfs"} {
puts "DFStype is aggcoldfs"
Mac/802_11 set max_count_inc 0

} elseif { $opt(dfstype) == "scaled802"} {
Mac/802_11 set cwmin_ $cw
puts "802 cwmin is $cw"
puts "type is Scaled 802.11"
} else { #802.11
puts "type is 802.11"
}

runtest

