/* Ported from CMU/Monarch's code, nov'98 -Padma
   wireless-mac-802_11.h 
   modified vaidya
 */
#ifndef ns_mac_80211_h
#define ns_mac_80211_h

#define TIMENOW (Scheduler::instance().clock())

#include <tamu_fairpatch.h>
#include <random.h>
#include <stdlib.h>

#include "tamu_mac-timers.h"
#include "marshall.h"

#define GET_ETHER_TYPE(x)		GET2BYTE((x))
#define SET_ETHER_TYPE(x,y)            {u_int16_t t = (y); STORE2BYTE(x,&t);}

//Seema: debugging macros
#define FIRE 
//{ printf("line %d \n",__LINE__); fflush(stdout); }
#define P(x) x
#include <math.h>

/* ======================================================================
   Frame Formats
   ====================================================================== */

#define	MAC_ProtocolVersion	0x00

#define MAC_Type_Management	0x00
#define MAC_Type_Control	0x01
#define MAC_Type_Data		0x02
#define MAC_Type_Reserved	0x03

#define MAC_Subtype_RTS		0x0B
#define MAC_Subtype_CTS		0x0C
#define MAC_Subtype_ACK		0x0D
#define MAC_Subtype_Data	0x00

struct frame_control {
	u_char		fc_subtype		: 4;
	u_char		fc_type			: 2;
	u_char		fc_protocol_version	: 2;

	u_char		fc_order		: 1;
	u_char		fc_wep			: 1;
	u_char		fc_more_data		: 1;
	u_char		fc_pwr_mgt		: 1;
	u_char		fc_retry		: 1;
	u_char		fc_more_frag		: 1;
	u_char		fc_from_ds		: 1;
	u_char		fc_to_ds		: 1;
};

struct rts_frame {
	struct frame_control	rf_fc;
	u_int16_t		rf_duration;
	u_char			rf_ra[ETHER_ADDR_LEN];
	u_char			rf_ta[ETHER_ADDR_LEN];
	u_char			rf_fcs[ETHER_FCS_LEN];
};


//Seema: send_tag field for virtual clock update deleted from cts,ack and 
//hdr_mac802_11 structures.

struct cts_frame {
	struct frame_control	cf_fc;
	u_int16_t		cf_duration;
	u_char			cf_ra[ETHER_ADDR_LEN];
	u_char			cf_fcs[ETHER_FCS_LEN];
};

struct ack_frame {
	struct frame_control	af_fc;
	u_int16_t		af_duration;
	u_char			af_ra[ETHER_ADDR_LEN];
	u_char			af_fcs[ETHER_FCS_LEN];
};

struct hdr_mac802_11 {
	struct frame_control	dh_fc;
	u_int16_t		dh_duration;
	u_char			dh_da[ETHER_ADDR_LEN];
	u_char			dh_sa[ETHER_ADDR_LEN];
	u_char			dh_bssid[ETHER_ADDR_LEN];
	u_int16_t		dh_scontrol;
	u_char			dh_body[0]; // XXX Non-ANSI ???

#ifdef EXPONEN_CW
  u_int32_t               cw_value; //contains delta for exp, sqrt mappings
#endif EXPONEN_CW

};


/* ======================================================================
   Definitions
   ====================================================================== */
#define ETHER_HDR_LEN				\
	((phymib_->PreambleLength >> 3) +	\
	 (phymib_->PLCPHeaderLength >> 3) +	\
	 sizeof(struct hdr_mac802_11) +		\
	 ETHER_FCS_LEN)

#define ETHER_RTS_LEN				\
	((phymib_->PreambleLength >> 3) +        \
         (phymib_->PLCPHeaderLength >> 3) +      \
	 sizeof(struct rts_frame))

#define ETHER_CTS_LEN				\
        ((phymib_->PreambleLength >> 3) +        \
         (phymib_->PLCPHeaderLength >> 3) +      \
         sizeof(struct cts_frame))

#define ETHER_ACK_LEN				\
        ((phymib_->PreambleLength >> 3) +        \
         (phymib_->PLCPHeaderLength >> 3) +      \
	 sizeof(struct ack_frame))

#define	RTS_Time	(8 * ETHER_RTS_LEN / bandwidth_)
#define CTS_Time	(8 * ETHER_CTS_LEN / bandwidth_)
#define ACK_Time	(8 * ETHER_ACK_LEN / bandwidth_)
#define DATA_Time(len)	(8 * (len) / bandwidth_)

#define MAC_MinPktSize  (512)
// measured in bytes

/*
 *  IEEE 802.11 Spec, section 9.2.5.7
 *	- After transmitting an RTS, a node waits CTSTimeout
 *	  seconds for a CTS.
 *
 *  IEEE 802.11 Spec, section 9.2.8
 *	- After transmitting DATA, a node waits ACKTimeout
 *	  seconds for an ACK.
 *
 *  IEEE 802.11 Spec, section 9.2.5.4
 *	- After hearing an RTS, a node waits NAVTimeout seconds
 *	  before resetting its NAV.  I've coined the variable
 *	  NAVTimeout.
 *
 */
#define CTSTimeout	((RTS_Time + CTS_Time) + 2 * sifs_)
#define ACKTimeout(len)	(DATA_Time(len) + ACK_Time + sifs_ + difs_)

#define NAVTimeout	(2 * phymib_->SIFSTime + CTS_Time + 2 * phymib_->SlotTime) 



#define RTS_DURATION(pkt)	\
	usec(sifs_ + CTS_Time + sifs_ + TX_Time(pkt) + sifs_ + ACK_Time)

#define CTS_DURATION(d)		\
	usec((d * 1e-6) - (CTS_Time + sifs_))

#define DATA_DURATION()		\
	usec(ACK_Time + sifs_)

#define ACK_DURATION()	0x00		// we're not doing fragments now

/*
 * IEEE 802.11 Spec, section 15.3.2
 *	- default values for the DSSS PHY MIB
 */
#define DSSS_CWMin			31
#define DSSS_CWMax			1023
#define DSSS_SlotTime			0.000020	// 20us
#define	DSSS_CCATime			0.000015	// 15us
#define DSSS_RxTxTurnaroundTime		0.000005	// 5us
#define DSSS_SIFSTime			0.000010	// 10us
#define DSSS_PreambleLength		144		// 144 bits
#define DSSS_PLCPHeaderLength		48		// 48 bits

class PHY_MIB {
public:
	u_int32_t	CWMin;
	u_int32_t	CWMax;
	double		SlotTime;
	double		CCATime;
	double		RxTxTurnaroundTime;
	double		SIFSTime;
	u_int32_t	PreambleLength;
	u_int32_t	PLCPHeaderLength;
};


/*
 * IEEE 802.11 Spec, section 11.4.4.2
 *      - default values for the MAC Attributes
 */
#define MAC_RTSThreshold		3000		// bytes
#define MAC_ShortRetryLimit		7		// retransmittions ???
#define MAC_LongRetryLimit		4		// retransmissions ???
#define MAC_FragmentationThreshold	2346		// bytes
#define MAC_MaxTransmitMSDULifetime	512		// time units
#define MAC_MaxReceiveLifetime		512		// time units

class MAC_MIB {
public:
	//		MACAddress;
	//		GroupAddresses;
	u_int32_t	RTSThreshold;
	u_int32_t	ShortRetryLimit;
	u_int32_t	LongRetryLimit;
	u_int32_t	FragmentationThreshold;
	u_int32_t	MaxTransmitMSDULifetime;
	u_int32_t	MaxReceiveLifetime;
	//		ManufacturerID;
	//		ProductID;

	u_int32_t	TransmittedFragmentCount;
	u_int32_t	MulticastTransmittedFrameCount;
	u_int32_t	FailedCount;	
	u_int32_t	RetryCount;
	u_int32_t	MultipleRetryCount;
	u_int32_t	FrameDuplicateCount;
	u_int32_t	RTSSuccessCount;
	u_int32_t	RTSFailureCount;
	u_int32_t	ACKFailureCount;
	u_int32_t	ReceivedFragmentCount;
	u_int32_t	MulticastReceivedFrameCount;
	u_int32_t	FCSErrorCount;
};


/* ======================================================================
   The following destination class is used for duplicate detection.
   ====================================================================== */
class Host {
public:
	LIST_ENTRY(Host) link;
	u_int32_t	index;
	u_int32_t	seqno;
};


/* ======================================================================
   The actual 802.11 MAC class.
   ====================================================================== */
class Mac802_11 : public Mac {
	friend class DeferTimer;
	friend class BackoffTimer;
	friend class IFTimer;
	friend class NavTimer;
	friend class RxTimer;
	friend class TxTimer;
public:
	Mac802_11(PHY_MIB* p, MAC_MIB *m);
        void		recv(Packet *p, Handler *h);
        inline int	hdr_dst(char* hdr, int dst = -2); //???
        inline int	hdr_src(char* hdr, int src = -2); //???
	inline int	hdr_type(char* hdr, u_int16_t type = 0);

protected:
	void	backoffHandler(void);
	void	deferHandler(void);
	void	navHandler(void);
	void	recvHandler(void);
	void	sendHandler(void);
	void	txHandler(void);

private:
	int		command(int argc, const char*const* argv);

        /* to implemend fair scheduling */
#ifdef FAIR_SCHEDULING
	double last_send_tag;
	double virtual_time;
	double weight;
	int count_inc;

	double randomizer;
	//Seema: Scaling factor eliminates the use of vt_multiplier
	//double vt_multiplier;
	double scaling_factor;
	int max_count_inc;
	int backoff_win;  //it is the CollisionWindow pmt discussed in paper

#ifdef EXPONEN_CW
	double exp_threshold;
	double k1_multiplier;
	double k2_multiplier;
#endif EXPONEN_CW

#endif FAIR_SCHEDULING
  

	/*
	 * Called by the timers.
	 */
	void		recv_timer(void);
	void		send_timer(void);
	int		check_pktCTRL();
	int		check_pktRTS();
	int		check_pktTx();

	/*
	 * Packet Transmission Functions.
	 */
	void	send(Packet *p, Handler *h);
	void 	sendRTS(int dst);
	void	sendCTS(int dst, double duration);
	void	sendACK(int dst);
	void	sendDATA(Packet *p);
	void	RetransmitRTS();
	void	RetransmitDATA();

	/*
	 * Packet Reception Functions.
	 */
	void	recvRTS(Packet *p);
	void	recvCTS(Packet *p);
	void	recvACK(Packet *p);
	void	recvDATA(Packet *p);

	void		capture(Packet *p);
	void		collision(Packet *p);
	void		discard(Packet *p, const char* why);
	void		rx_resume(void);
	void		tx_resume(void);

	inline int	is_idle(void);

	/*
	 * Debugging Functions.
	 */
	void		trace_pkt(Packet *p);
	void		dump(char* fname);

	inline int initialized() {
		return (phymib_ && macmib_ && cache_ && logtarget_ && Mac::initialized());
	}

	void mac_log(Packet *p) {
		logtarget_->recv(p, (Handler*) 0);
	}

	inline double TX_Time(Packet *p) {
		double t = DATA_Time((HDR_CMN(p))->size());
		
		if(t < 0.0) {
			drop(p, "XXX");
			exit(1);
		}
		return t;
	}


#ifdef EXT_ERRORS
	int put_exterrors(int size) {
	  
	  //check if this flow is subject to errors or not
	  if (useerr == 0)
	    return 0;

	  //choose error rate b/w low and high and normalize
	  //error model used for the thesis
	  double ran1 = Random::uniform(errratelow,errratehigh)*(double)size/MAC_MinPktSize;
	  
	  double ran2 = Random::uniform();
	  if (ran2 <= ran1)
	    return 1;
	  else
	    return 0;

	  /*
	    //Use this for byte based error rate, where errratelow is the 
	    // probability that a byte is in error.
	    double p=errratelow;
	    double ds = (double) size;
	    
	    double prob = 1.0 - pow((1-p),ds);
	    P(printf("err is %g size is %g prob is %g\n",p,ds,prob););
	    P(fflush(stdout););
	    double ranvar = Random::uniform();
	    if (ranvar <= prob)
	    return 1;
	    else
	    return 0;
	  */
	}

#endif EXT_ERRORS


#ifdef FAIR_SCHEDULING
//This method returns the smallest prime greater than key using binary search
int get_prime(int key) {
	  
  int top,bot,mid;
  int a[] = {5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97,101,103,107,109,113,127,131,137,139,149,151,157,163,167,173,179,181,191,193,197,199,211,223,227,229,233,239,241,251,257,263,269,271,277,281,283,293,307,311, 313,317,331,337,347,349,353,359,367,373,379,383,389,397};

  top=0;
  bot = 75;
  
  if (key < a[top])
    return (a[top]);
  if (key > a[bot])
    return (a[bot]);

  mid = (top+bot)/2;   
  while (top < bot) {
    if (a[mid] == key)
      return (key);
    if (a[mid] > key) 
      bot = mid -1;
    else
      top = mid +1;
    mid = (top+bot)/2;
  }
  return (a[mid]);
    
}      	
#endif FAIR_SCHEDULING

//Function for collision resolution
inline void inc_cw() {
#ifdef FAIR_SCHEDULING
  //intcw is used by Aggressive_DFS only
  static int intcw = 0;
  count_inc++;
  
                if (count_inc > max_count_inc) { 
		  if (max_count_inc == 0) {
		    // this is Aggressive_DFS
		    if (count_inc == 1) 
		      intcw = 2*cw_+1;
		    else
		      intcw = 2*intcw+1;
		    if (intcw > phymib_->CWMax)
		      intcw = phymib_->CWMax;
		    cw_ = 1+(Random::random()%intcw);
		      
		  } else { 
		    cw_ = (cw_ << 1) + 1;
		    if(cw_ > phymib_->CWMax)
		      cw_ = phymib_->CWMax;
		    
		  }
		}
                else {
		  // This is regular DFS code
		  double tempd = pow(2.0,((double)count_inc-1.0));
                  tempd *= (double) backoff_win;
		  //tempd += 1.0;
		  
		  cw_ = 1+(Random::random()%((int) tempd));
				  		    
		}
		//@@@
		printf("%g incnode %d cw %d\n",TIMENOW,index_,cw_);
		fflush(stdout);
		
		
#ifdef EXPONEN_CW
		P(printf("inccw %d linear_cw was %d cw %d\n",index_,linear_cw,cw_););
		P(fflush(stdout););
		linear_cw=cw_;

		/*if (pktTx_ != 0) {
		  struct hdr_mac802_11* dh = HDR_MAC802_11(pktTx_);
		  dh->cw_value = linear_cw;
		  }*/
#endif EXPONEN_CW
                 
#else
                 
                cw_ = (cw_ << 1) + 1;
                if(cw_ > phymib_->CWMax)
                        cw_ = phymib_->CWMax;
#endif FAIR_SCHEDULING
		 
#ifdef FAIRDEBUG
		printf("%g %d inc_cw to cw = %d\n",TIMENOW,index_,cw_); 
#endif FAIRDEBUG

#ifdef DEBUG802
		printf("%g %d inc_cw to cw = %d\n",TIMENOW,index_,cw_); 
#endif DEBUG802
	}

  inline void rst_cw() { 
#ifdef FAIR_SCHEDULING
     
		count_inc = 0;
#ifdef EXPONEN_CW
		linear_cw = phymib_->CWMin;
#endif EXPONEN_CW
#endif FAIR_SCHEDULING
		 
		cw_ = phymib_->CWMin;

  }

	inline u_int16_t usec(double t) {
		u_int16_t us = (u_int16_t)ceil(t *= 1e6);
		return us;
		
	}


	inline void set_nav(u_int16_t us) {
		double now = Scheduler::instance().clock();
		double t = us * 1e-6;

		if((now + t) > nav_) {
			nav_ = now + t;
			if(mhNav_.busy())
				mhNav_.stop();
			mhNav_.start(t);
		}
	}


#ifdef FAIR_SCHEDULING
#ifdef EXPONEN_CW
	

	void expomap_setcw(int val) {
	  
	  assert(val>0);
	  if (val <= 0) {
	    printf("linear_cw <= 0\n");
	    //inc_cw();
	    exit(0);
	  }
	  
	  linear_cw = val;
	  if (val >= (int) exp_threshold) {
	    //uncomment this for exponential mapping
	    double t1 =exp_threshold+exp_threshold*(1-exp(-1.0*((double)val-exp_threshold)*k2_multiplier));
	    
	    //uncomment this for square-root mapping
	    //double t1 = k1_multiplier*sqrt(val) + 1.0;
	    cw_ = (int) t1;
	    
	  } else {
	    cw_ = val;
	  }

	  
	  P(printf("node %d linear cw %d expo cw %d\n",index_,val,cw_););
	  P(fflush(stdout););
	}
#endif EXPONEN_CW
#endif FAIR_SCHEDULING



#ifdef FAIR_SCHEDULING
//This 	function is used for assigning the INITIAL backoff interval
	void update_cw() {
	   
#ifdef FAIRDEBUG
		printf("update_cw: %d ",index_); 
#endif FAIRDEBUG
		count_inc = 0 ;

		double tempdiff = last_send_tag - virtual_time;

        	if (last_send_tag <= virtual_time) {
			inc_cw();
                 	//printf(" last_send_tag %f <= virtual_time %f : ", \
		 	//last_send_tag,virtual_time);
        	}
		else {

		  //double tempd =vt_multiplier*(last_send_tag - virtual_time);
		  double tempd = (last_send_tag - virtual_time);
		  tempd *= ((Random::uniform(-0.5,+0.5))*randomizer + 1.0);
		  int tempi = (int) tempd;
		  
#ifdef EXPONEN_CW
		  expomap_setcw(tempi);
#else
		  cw_ = tempi;
		  
#endif EXPONEN_CW	
		  P(printf("%d updatecw cw = %d\n",index_,cw_););
		  
		}
		  
#ifdef FAIRDEBUG
		printf("cw = %d\n",cw_); 
		fflush(stdout);
#endif FAIRDEBUG
			
	}

#endif FAIR_SCHEDULING



protected:
	PHY_MIB		*phymib_;
	MAC_MIB		*macmib_;

private:
	/*
	 * Mac Timers
	 */
	IFTimer		mhIF_;		// interface timer
	NavTimer	mhNav_;		// NAV timer
	RxTimer		mhRecv_;		// incoming packets
	TxTimer		mhSend_;		// outgoing packets

	DeferTimer	mhDefer_;	// defer timer
	BackoffTimer	mhBackoff_;	// backoff timer

	/* ============================================================
	   Internal MAC State
	   ============================================================ */
	double		nav_;		// Network Allocation Vector

	MacState	rx_state_;	// incoming state (MAC_RECV or MAC_IDLE)
	MacState	tx_state_;	// outgoint state
#if 1
	int		tx_active_;	// transmitter is ACTIVE
#endif

	Packet		*pktRTS_;	// outgoing RTS packet
	Packet		*pktCTRL_;	// outgoing non-RTS packet

	u_int32_t	cw_;		// Contention Window

#ifdef EXPONEN_CW

	u_int32_t         linear_cw;
#endif EXPONEN_CW


	u_int32_t	ssrc_;		// STA Short Retry Count
	u_int32_t	slrc_;		// STA Long Retry Count
	double		sifs_;		// Short Interface Space
	double		pifs_;		// PCF Interframe Space
	double		difs_;		// DCF Interframe Space
	double		eifs_;		// Extended Interframe Space
	double		tx_sifs_;
	double		tx_pifs_;
	double		tx_difs_;

	int		min_frame_len_;

        NsObject*	logtarget_;

        
        u_int16_t	sta_seqno_;  //duplication detection state
	int		cache_node_count_;
	Host		*cache_;


	//Error Model parameters
#ifdef EXT_ERRORS
	int             useerr;     // whether to apply errors or not
	double          errratelow; // lower limit of error model
        double          errratehigh;// high limit of the error model
	int             nodesinerr;  // number of nodes in error
#endif EXT_ERRORS


	// Fair scheduling specific error resolution parameters
#ifdef FAIR_SCHEDULING
#ifdef EXT_ERRORS

	// These variables are fixed in the beginning and do not change later
	
        int             resolveerr; // whether to resolve errors or not
        
        double          orgweight;  // original weight of the flow
	double          powerfactor;// power factor
	double          alpha;      // error averaging factor

	// These variables change dynamically during simulation

	int             pktspererr; // number of packets per error
	double          esterrrate; // estimated error rate
	int             lag;        // observed lag of the flow
	
#endif EXT_ERRORS
#endif FAIR_SCHEDULING

};


#endif /* __mac_80211_h__ */

